/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

package org.tribblix.illuminate.explorer;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import org.tribblix.illuminate.InfoCommand;
import uk.co.petertribble.jkstat.api.JKstat;
import uk.co.petertribble.jkstat.api.Kstat;
import uk.co.petertribble.jkstat.gui.AccessoryNetPanel;
import uk.co.petertribble.jkstat.gui.KstatAccessoryPanel;
import uk.co.petertribble.jkstat.gui.KstatAccessorySet;
import uk.co.petertribble.jkstat.gui.SparkRateAccessory;

/**
 * ZoneInfoPanel - shows Zone information.
 * @author Peter Tribble
 * @version 1.0
 */
public class ZoneInfoPanel extends InfoPanel {

    private JKstat jkstat;
    private KstatAccessorySet kas;
    private List <KstatAccessoryPanel> kaplist;

    /**
     * Display a zone information panel.
     *
     * @param hi The item to display
     * @param jkstat A JKstat object
     */
    public ZoneInfoPanel(SysItem hi, JKstat jkstat) {
	super(hi);
	this.jkstat = jkstat;
	kaplist = new ArrayList <KstatAccessoryPanel> ();

	if (hi.getType() == SysItem.ZONE_CONTAINER) {
	    displaySummary();
	} else if (hi.getType() == SysItem.ZONE_ZONE) {
	    displayZone();
	} else if (hi.getType() == SysItem.ZONE_PROC) {
	    displayZoneProc();
	} else if (hi.getType() == SysItem.ZONE_NET) {
	    displayZoneNet();
	}

	validate();
	kas = new KstatAccessorySet(kaplist, 1);
    }

    @Override
    public void stopLoop() {
	kas.stopLoop();
    }

    /*
     * Top level summary. List of Zones.
     */
    private void displaySummary() {
	jvp.add(new JLabel("Zone Summary"));
	addText(new CommandTableModel(new InfoCommand("za", "/usr/sbin/zoneadm",
					"list -icv")));
    }

    /*
     * Describe a Zone.
     */
    private void displayZone() {
	ZoneEntry ze = (ZoneEntry) hi.getAttribute("zoneentry");
	jvp.add(new JLabel("Details of "+ze.getState()+" zone "+ze.getName()));
	addText(ze.getConfig());
    }

    /*
     * Show Zone processes
     */
    private void displayZoneProc() {
	jvp.add(new ProcessInfoPanel(hi));
    }

    /*
     * Show Zone network
     */
    private void displayZoneNet() {
	ZoneEntry ze = (ZoneEntry) hi.getAttribute("zoneentry");
	@SuppressWarnings("unchecked")
	Map <String, Kstat> netMap =
	    (Map <String, Kstat>) hi.getAttribute("netmap");
	for (ZoneNet znet : ze.getNetworks()) {
	    Kstat ks = netMap.get(znet.getPhysical());
	    if (ks == null) {
		addText("Interface " + znet.getPhysical());
	    } else {
		// outer panel
		JPanel opanel = new JPanel();
		opanel.setLayout(new BoxLayout(opanel, BoxLayout.PAGE_AXIS));

		// regular network accessory
		AccessoryNetPanel acp = new AccessoryNetPanel(ks, 5, jkstat);
		kaplist.add(acp);
		opanel.add(acp);

		// panel for row of sparkcharts
		JPanel npanel = new JPanel();
		npanel.add(new JLabel("In: "));
		SparkRateAccessory kap =
		    new SparkRateAccessory(ks, -1, jkstat, "rbytes64");
		kap.enableTips("Current kb/s in:", 1.0/1024.0);
		npanel.add(kap);
		kaplist.add(kap);
		npanel.add(new JLabel("  Out: "));
		kap = new SparkRateAccessory(ks, -1, jkstat, "obytes64");
		kap.enableTips("Current kb/s out:", 1.0/1024.0);
		npanel.add(kap);
		kaplist.add(kap);
		opanel.add(npanel);
		opanel.setBorder(BorderFactory.createTitledBorder
				 ("Interface " + ks.getName()));
		jvp.add(opanel);
	    }
	}
    }
}
